.rc-slider {
  flex: auto;
  height: 24px;
  margin: 4px -12px 0 -12px;
}

.rc-slider-rail,
.rc-slider-track {
  background-color: var(--slider-background-color);
  height: 2px;
}

.rc-slider-track {
  background-color: var(--fill-on-active);
}

.rc-slider-dot {
  background-color: var(--slider-background-color);
  border: var(--field-enabled);
  bottom: 0;
  height: 6px;
  width: 6px;
}

.rc-slider-dot-active {
  background-color: var(--fill-on-active);
}

.rc-slider-handle,
.rc-slider-handle:hover,
.rc-slider-handle-dragging.rc-slider-handle-dragging.rc-slider-handle-dragging,
.rc-slider-handle-click-focused:focus {
  background-color: var(--surfaces-bg-01);
  border: solid 6px var(--fill-on-active);
  border-color: var(--fill-on-active);
  border-radius: 100%;
  height: 16px;
  margin: -8px 0 0 0;
  position: absolute;
  width: 16px;
}

.rc-slider-handle:hover,
.rc-slider-handle-dragging.rc-slider-handle-dragging.rc-slider-handle-dragging {
  border: solid 4px var(--fill-on-active);
  box-shadow: unset;
  cursor: context-menu;
}

.rc-slider-handle-click-focused:focus {
  box-shadow: unset;
}

.rc-slider-handle:active {
  border: solid 6px var(--fill-on-active);
  box-shadow: unset;
  cursor: grabbing;
  height: 16px;
  margin-top: -8px;
  width: 16px;
}

.rc-slider-mark-text {
  color: var(--text-secondary);
  font-size: var(--text-size-01);
  margin-top: var(--spacing-01);
  min-width: 44px;
}

.rc-slider-mark-text-active {
  color: var(--fill-on-active);
}

.slider_control,
.slider_control_no_space,
.range_slider_control,
.range_slider_control_no_space {
  display: flex;
  height: 20px;
  margin: 0 8px 24px -4px;
  width: 100%;
}

.slider_control_no_space {
  margin-bottom: -4px;
}

.range_slider_control_no_space {
  margin: 2px 8px -4px 0px;
  padding: 0 30px 25px 30px !important;
}
.range_slider_control {
  padding: 0 30px 25px 30px !important;
}

.slider_input_field_left,
.slider_input_field_right,
.slider_input_field_no_space_left,
.slider_input_field_no_space_right {
  background-color: inherit;
  font-size: var(--text-size-02);
  margin-right: 4px;
  padding-bottom: var(--spacing-08);
  text-decoration: underline;
  text-decoration-color: var(--border-subtle-alpha-01);
  max-width: 54px;
  min-width: 40px;
}

.slider_input_field_right,
.slider_input_field_no_space_right {
  margin-right: 0;
  text-align: right;
}

.slider_input_field_no_space_left,
.slider_input_field_no_space_right {
  padding-bottom: 4px;
}

.slider_inner_container,
.range_slider_inner_container {
  align-items: center;
  display: flex;
  flex-direction: row;
}

.range_slider_inner_container {
  flex-direction: column;
}

.slider_input_container {
  display: flex;
  justify-content: space-between;
  width: 100%;
}
