input[type="checkbox" i] {
  align-items: center;
  -webkit-appearance: none;
  border: 1px solid var(--text-secondary);
  display: inline-flex;
  height: 16px;
  margin: 4px 8px 4px 4px;
  vertical-align: middle;
  width: 16px;
}

input[type="checkbox"]:checked:before {
  color: var(--text-primary);
  content: "✓";
  display: flex;
  justify-content: center;
  margin-bottom: 1px;
  margin-left: 1px;
}

input[type="checkbox"]:hover {
  border: 1px solid var(--text-primary);
}

input[type="checkbox"]:focus {
  border: 1px solid var(--text-primary);
  box-shadow: 0 0 0 2px var(--text-secondary);
}
